from ipaddress import ip_address, ip_network

##########################################################################################################
#
# Independent functions used multiple times from different program parts
#
##########################################################################################################


def find_string_between(queue_logging, thread_num, string_to_search, start_string, end_string):
    """
    :param queue_logging: queue for logging (multiprocessing compatible) -> logging in one file NaviX.log
    :param thread_num: used for identifying process in logs
    :param string_to_search: given input string to find substring
    :param start_string: start substring to search for in given string, for, cut string after
    :param end_string: end substring to search for in given string, cut string before
    :return: given string between given substrings
    function to find string between two specified substrings
    """
    try:
        start = string_to_search.index(start_string) + len(start_string)
        end = string_to_search.index(end_string, start)
        return string_to_search[start:end]
    except ValueError:
        queue_logging.put("DEBUG: " + str(thread_num) + ": No string found between given substrings")
        return ""


def remove_blanks_from_string(given_string):
    """
    :param given_string: input string from which the blanks should be removed
    :return: input string without blanks
    removes blanks from a string
    """
    return given_string.replace(" ", "")


def is_ip_in_network_range(ip, range_to_look):
    """
    :param ip: ip to check if in given range
    :param range_to_look: range to check against the ip
    :return: returns True or False
    checks if an IP is in a given range
    """
    return ip_address(ip) in ip_network(range_to_look)
