from time import time
from random import choice


def check_waiting_time_and_add_resources(queue_logging, thread_num, server_dict, last_time, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param server_dict:
    :param last_time:
    :param settings:
    :return:
    this function evaluates how long a job is waiting since first call of the translate hook
    if the job has longer waited than specified in self.config and/or NaviX.cnf the job gets a random
    server added with highest weight
    """
    no_further_server = False
    try:
        if last_time and \
                ((time() - float(last_time)) > settings.time_before_adding_additional_resources):

            queue_logging.put("DEBUG: " + str(thread_num) + ": Jobs waits longer than "
                              + str(settings.time_before_adding_additional_resources)
                              + " seconds, adding another server!")
            i = 1
            try:
                while (((time() - float(last_time)) > settings.time_before_adding_additional_resources * i)
                       and not no_further_server):
                    try:
                        get_random_server_with_highest_weight(queue_logging, thread_num, settings, server_dict)
                    except Exception as e:
                        queue_logging.put("ERROR: " + str(thread_num)
                                          + ": Failed while trying to append random server "
                                          + "with highest weight -> Abort!"
                                          + "\nPython-ERROR: " + str(e))
                        raise ValueError(str(e))
                    i += 1
            except Exception as e:
                queue_logging.put("ERROR: " + str(thread_num)
                                  + ": Failed while checking job waiting time, "
                                  + "could not evaluate while condition -> Abort!"
                                  + "\nPython-ERROR: " + str(e))
                raise ValueError(str(e))
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Failed while checking job waiting time -> Abort!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))


def get_random_server_with_highest_weight(queue_logging, thread_num, settings, server_dict):
    """
    :param :
    :return:
    this manages the selection of a random server, servers with higher weight are preferred,
    only calls sub functions
    """
    try:
        local_known_servers = settings.known_servers.copy()
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to create local copy to ram of known servers!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    # remove already listed servers
    try:
        local_known_servers = remove_listed_server_from_local_copy(queue_logging, thread_num, server_dict,
                                                                   local_known_servers)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num) + ": failed to remove servers from local list!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    # set default weight if server have no weight
    try:
        local_known_servers = set_default_weight(queue_logging, thread_num, local_known_servers, settings)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to set default weight to every server without weight in local list!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    # get servers with highest rank
    try:
        weight = get_servers_with_highest_rank(queue_logging, thread_num, local_known_servers)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to get servers with highest rank from local list!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    try:
        local_known_servers = remove_servers_with_lower_weight(queue_logging, thread_num, local_known_servers, weight)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to remove servers with lower rank rank from local list!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    try:
        server_dict = select_random_server_from_dict(queue_logging, thread_num, local_known_servers, server_dict,
                                                     settings)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to select random servers from local dictionary!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    return server_dict


def remove_listed_server_from_local_copy(queue_logging, thread_num, server_dict, local_known_servers):
    """
    :param queue_logging:
    :param thread_num:
    :param server_dict:
    :param local_known_servers:
    :return:
    removes listed server in server_dict in a copy of all server given in 'known_hosts.yaml'
    this is required to e.g. add further random server
    """
    try:
        for server, values in server_dict.items():
            if values.get('IP') is not None:
                del local_known_servers[values.get('IP')]
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to remove server from local_known_server dict which are already"
                          + "listed for HPDA_RANK!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    return local_known_servers


def set_default_weight(queue_logging, thread_num, local_known_servers, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param local_known_servers:
    :param settings:
    :return:
    if a server has no specific weight it gets an default weight, specified in self.config and/or NaviX.cnf
    the weight is only relevant for the ranking
    """
    try:
        for server, values in iter(local_known_servers.items()):
            if values.get('weight') is None:
                local_known_servers[server]['weight'] = settings.server_weight
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num) + ": failed to set default weight to "
                          + "servers without explicit weight!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    return local_known_servers


def remove_servers_with_lower_weight(queue_logging, thread_num, local_known_servers, weight):
    """
    :param queue_logging:
    :param thread_num:
    :param local_known_servers:
    :param weight:
    :return:
    removes all servers in local_known_servers with a weight lower than given
    """
    server_to_remove = []
    try:
        for server, values in iter(local_known_servers.items()):
            if int(values.get('weight')) < weight:
                server_to_remove.append(server)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to set up list to remove servers from local list due low weight!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    try:
        for element in server_to_remove:
            del local_known_servers[element]
        return local_known_servers
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to remove servers from local list due low weight!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))


def get_servers_with_highest_rank(queue_logging, thread_num, local_known_servers):
    """
    :param queue_logging:
    :param thread_num:
    :param local_known_servers:
    :return:
    this returns the highest weight out of all server in local_known_servers, this is used e.g. to append a server
    to server_dict, if the jobs already waited to long
    """
    weight = 0
    try:
        for server, values in iter(local_known_servers.items()):
            if int(values.get('weight')) > weight:
                weight = int(values.get('weight'))
        return weight
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to get highest weight of \"unused\" servers!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))


def select_random_server_from_dict(queue_logging, thread_num, local_known_servers, server_dict, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param local_known_servers:
    :param server_dict:
    :param settings:
    :return:
    this chooses random a server out of local_known servers (e.g. append server after time X)
    """
    try:
        if local_known_servers:
            random_server = choice(list(local_known_servers))
        else:
            queue_logging.put("WARNI: " + str(thread_num)
                              + ": No further server to add to requirements!")
            return server_dict
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to pick random servers from local list!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    try:
        server_dict = known_host_to_server_dict(local_known_servers.get(random_server),
                                                random_server, server_dict, settings)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": failed to add random server to HPDA-list (server-dict)! "
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))
    return server_dict


def known_host_to_server_dict(server, ip, server_dict, settings):
    """
    :param server:
    :param ip:
    :param server_dict:
    :param settings:
    :return:
    gets an object server (known host) and add the server to the thread object server_dict
    if the server is already in server_dict the count of it is increased by one (higher count = higher rank)
    """
    if server.get('name') in server_dict:
        server_dict[server.get('name')]['count'] += 1
    else:
        server_dict[server.get('name')] = {}
        server_dict[server.get('name')]['count'] = 1
        server_dict[server.get('name')]['IP'] = ip
        if server.get('weight') is not None:
            server_dict[server.get('name')]['weight'] = server.get('weight')
        else:
            server_dict[server.get('name')]['weight'] = settings.server_weight
    return server_dict
