from time import time


def create_hpda_rank(queue_logging, thread_num, server_dict, count_found_files, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param server_dict:
    :param count_found_files:
    :param settings
    :return:
    creates a string which will be used as rank
    """
    hpda_rank = ""
    try:
        monitor_rank_server = []
        # prevent dividing by zero if no file is found!
        if count_found_files == 0:
            queue_logging.put("DEBUG: " + str(thread_num)
                              + ": No valid addresses found for files, set HPDA_RANK to (0)")
            hpda_rank = settings.hpda_rank + " = (0)\n"
        else:
            for server, value in iter(server_dict.items()):
                hpda_rank += "(" + str(server) + ") *" \
                                        + str(int((int(value.get('count'))/count_found_files) *
                                              int(value.get('weight')))) \
                                        + " + "
                monitor_rank_server.append(monitoring_rank(str(server), str(value.get('count')), str(value.get('IP')),
                                           str(count_found_files), str(value.get('weight'))))

            # remove last ' + '
            queue_logging.put("DEBUG: " + str(thread_num)
                              + ": Address for files: " + str(hpda_rank[:-3]))
            hpda_rank = settings.hpda_rank + " = " + hpda_rank[:-3] + "\n"
        return hpda_rank, monitor_rank_server

    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Failed preparing cast of 'HPDA_RANK' classAd! -> Abort!"
                          + "\nServers to cast to classAd: \n"
                          + str(server_dict) + "\n"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))


def monitoring_rank(server, count, ip, count_found_files, weight):
    monitor_rank_server = dict()
    monitor_rank_server[server] = {}
    monitor_rank_server[server]['files_found'] = count_found_files
    monitor_rank_server[server]['count'] = count
    monitor_rank_server[server]['IP'] = ip
    monitor_rank_server[server]['weight'] = weight
    return monitor_rank_server


def add_timestamp(queue_logging, thread_num, reply, last_time, settings):
    """
    :param settings:
    :param queue_logging:
    :param thread_num:
    :param reply:
    :param last_time:
    :return:
    adds an timestamp to a reply to the hook
    but only if there is no timestamp till now
    """
    try:
        # if there was not time stamp - add one!
        if not last_time:
            time_now = str(time())
            reply += settings.hpda_timestamp + " = \"" + time_now + "\"\n"
            queue_logging.put("DEBUG: " + str(thread_num) + ": Timestamp added to reply")
        return reply
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Failed to append time stamp to classAd! -> Abort!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))


def add_hpda_requirement(queue_logging, thread_num, count_found_files, server_dict, last_time, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param count_found_files:
    :param server_dict:
    :param last_time: if it is None
                        => translate hook,
                      else it should be the timestamp from the first interaction with NaviX
                        => update hook
    :param settings:
    :return:
    adds to reply to the hook the "HPDA_REQUIREMENT" string
    """
    requirement_string = ""
    try:
        # prevent dividing by zero if no file is found!
        if count_found_files == 0:  # TODO or no_further_server or str(location_string) == "" necessary?
            # TODO maybe only servers above a specified rank should be added in the first round?
            requirement_string = "(True)"
        else:
            for server, value in iter(server_dict.items()):
                # First round only add files with more than e.g 50% cached files (specified in settings)
                if not last_time and \
                        ((int(value.get('count'))/count_found_files) >
                         settings.cached_percentage_before_adding_to_requirement):
                    requirement_string += "( " + str(server) + " )" + " || "
                elif last_time:
                    requirement_string += "( " + str(server) + " )" + " || "
            # remove last ' + '
            requirement_string = requirement_string[:-4]
        requirement_string = settings.hpda_requirement + " = " + requirement_string + "\n"
        return requirement_string
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Failed while casting 'HPDA_REQUIREMENT' classAd! -> Abort!"
                          + "\nServers to cast to classAd: \n"
                          + str(server_dict) + "\n"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError(str(e))


def add_classad_for_monitoring_to_reply(settings, reply, files_server_dict, files_to_search_array, monitor_rank_server):
    reply += (settings.file_found_on_server + " = \"" + str(files_server_dict) + "\"\n")
    reply += (settings.requested_files_mon + " = \"" + str(files_to_search_array) + "\"\n")
    reply += (settings.hpda_rank_monitoring + " = \"" + str(monitor_rank_server).replace("\"", "\\\"") + "\"\n")
    return reply
