from navix.request_pyxrootd import deeplocate_for_files_pyxrootd
from navix.common import find_string_between, is_ip_in_network_range


def get_location_of_files(queue_logging, thread_num, files_to_search_array, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param files_to_search_array:
    :param settings:
    :return:
    request location of files given in a list via pyxrootd from the caching management server
    """
    # if there is a file to look for, search them and give server IPs (V6) back
    if files_to_search_array:
        try:
            queue_logging.put("DEBUG: " + str(thread_num) + ": Searching over pyxrootd for: "
                              + str(files_to_search_array))
            return search_for_files_pyxrootd(queue_logging, thread_num, files_to_search_array, settings)
        except Exception as e:
            queue_logging.put("ERROR: " + str(thread_num)
                              + ": Something went wrong while searching for files with pyxrootd!\n-> abort"
                              + "\nPython-ERROR: " + str(e))
            raise ValueError
    else:
        queue_logging.put("WARNI: " + str(thread_num) + ": No files to search for -> no files to cache!")


def search_for_files_pyxrootd(queue_logging, thread_num, files_to_search_array, settings):
    """
    :param queue_logging:
    :param thread_num:
    :param files_to_search_array:
    :param settings:
    :return:
    asks CMS server for location of the requested files and writes the servers into a dictionary
    """
    count_found_files = 0
    server_dict = dict()
    files_monitor_dict = dict()
    # open xrootd client to check for specified files
    try:
        file_dict = deeplocate_for_files_pyxrootd(queue_logging, thread_num, settings.xrootd_server_address,
                                                  files_to_search_array, settings.pyxrootd_timeout)
    except Exception as err:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Python-ERROR: " + str(err)
                          + "\n -> Abort")
        raise ValueError(err)

    for file, ip_list in iter(file_dict.items()):
        found_on_server = False
        for ip in ip_list:
            if ip != "No server":
                found_on_server = True
                add_file_to_dict_for_monitoring_classad(file, ip, files_monitor_dict)
                queue_logging.put("DEBUG: " + str(thread_num)
                                  + ": Try to resolve ip to specified server name")
                # check if given ip can be resolved to config given name
                try:
                    queue_logging.put("DEBUG: " + str(thread_num)
                                      + " Given ip for lookup in known hosts:"
                                      + "\nip: " + str(ip)
                                      + "\nserver_dict before lookup of ip: " + str(server_dict))
                    try:
                        server_dict = find_server_to_location(queue_logging, thread_num, ip, settings, server_dict)
                    except ValueError:
                        queue_logging.put("WARNI: " + str(thread_num)
                                          + ": Server " + str(ip)
                                          + " not found in 'known_servers.yml'")
                except Exception as e:
                    queue_logging.put("ERROR: " + str(thread_num)
                                      + ": Unexpected error while trying to resolve ip to "
                                      + "HTCondor specified server name! -> abort"
                                      + "\nPython-ERROR: " + str(e))
                    raise ValueError
        if not found_on_server:
            add_file_to_dict_for_monitoring_classad(file, "No server", files_monitor_dict)
        else:
            count_found_files += 1

    return count_found_files, server_dict, files_monitor_dict


def find_server_to_location(queue_logging, thread_num, ip, settings, server_dict):
    """
    :param queue_logging:
    :param thread_num:
    :param ip:
    :param settings:
    :param server_dict:
    :return:
    check if received IPs from pyxrootd belong to a known server side (based on input in known_host.yml file)
    """
    server_found = False
    for IP_range, server in iter(settings.known_servers.items()):
        if is_ip_in_network_range(str((find_string_between(queue_logging, thread_num, ip, "[", "]"))), IP_range):
            server_found = True
            queue_logging.put("DEBUG: " + str(thread_num) + ": IP: "
                              + str((find_string_between(queue_logging, thread_num, ip, "[", "]")))
                              + " found in IP range: " + str(IP_range))
            # count how many files access each server side
            server_dict = known_host_to_server_dict(server, IP_range, settings, server_dict)

    if not server_found:
        queue_logging.put("WARNI: " + str(thread_num)
                          + ": Server " + str(ip)
                          + " not found in 'known_servers.yml'")

    queue_logging.put("DEBUG: " + str(thread_num)
                      + ": server names, with file count and weight:  " + str(server_dict))
    return server_dict


def known_host_to_server_dict(server, ip, settings, server_dict):
    """
    :param server:
    :param ip:
    :param settings:
    :param server_dict:
    :return:
    gets an object server (known host) and add the server to the thread object server_dict
    if the server is already in server_dict the count of it is increased by one (higher count = higher rank)
    """
    if server.get('name') in server_dict:
        server_dict[server.get('name')]['count'] += 1
    else:
        server_dict[server.get('name')] = {}
        server_dict[server.get('name')]['count'] = 1
        server_dict[server.get('name')]['IP'] = ip
        if server.get('weight') is not None:
            server_dict[server.get('name')]['weight'] = server.get('weight')
        else:
            server_dict[server.get('name')]['weight'] = settings.server_weight
    return server_dict


def add_file_to_dict_for_monitoring_classad(file_name, server_ip, files_server_dict):
    if file_name in files_server_dict:
        files_server_dict[str(file_name)].append(str(server_ip))
    else:
        files_server_dict[str(file_name)] = [str(server_ip)]
    return
