from XRootD import client
from XRootD.client.flags import OpenFlags


def create_pyxrootd_client(queue_logging, thread_num, server_address):
    ###########################################################################################
    #
    # Establish connection to xrootd manager
    #
    ###########################################################################################
    queue_logging.put("DEBUG: " + str(thread_num) + ": Starting client connection to xrootd")
    try:
        client_xrootd = client.FileSystem(server_address)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num) + ": Failed starting client connection to xrootd"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError("Failed to create pyxrootd instance to server: " + str(server_address))
    queue_logging.put("DEBUG: " + str(thread_num) + ": Connected successfully to xrootd")
    return client_xrootd


def deeplocate_for_files_pyxrootd(queue_logging, thread_num,
                                  server_address, files_to_search, pyxrootd_timeout):

    client_xrootd = create_pyxrootd_client(queue_logging, thread_num, server_address)
    ###########################################################################################
    #
    # Actual request of data location
    #
    ###########################################################################################
    file_dict = dict()
    for file_to_search in files_to_search:
        status = ""
        try:
            queue_logging.put("DEBUG: " + str(thread_num) + ": Requesting file "
                              + str(file_to_search) + " from xrootd")
            status, locations = client_xrootd.deeplocate(file_to_search, OpenFlags.REFRESH,
                                                         timeout=pyxrootd_timeout)
        except Exception as e:
            queue_logging.put("WARNI: " + str(thread_num)
                              + ": Requesting file "
                              + str(file_to_search)
                              + " from xrootd, lead to an error!"
                              + " Returned status from xrootd: " + str(status)
                              + "\n Proceeding with other files!"
                              + "\nPython-ERROR: " + str(e))
            status = "FAIL"

        if file_to_search not in file_dict:
            file_dict[file_to_search] = []
        if str(status) == "[SUCCESS] ":
            for location in locations:
                file_dict[file_to_search].append(location.address)
                queue_logging.put("DEBUG: " + str(thread_num)
                                  + ": File: " + file_to_search + " found on xrootd server.")
        else:
            file_dict[file_to_search].append("No server")
            queue_logging.put("DEBUG: " + str(thread_num)
                              + ": File: " + file_to_search + " NOT found on xrootd server. "
                              + "Status is: " + str(status))
    del client_xrootd
    return file_dict


def stat_for_files_pyxrootd(xrootd_client, file):
    ###########################################################################################
    #
    # Actual request of data statistics (mainly file size)
    #
    ###########################################################################################
    status, stat = xrootd_client.stat(file)
    return status, stat
