#!/bin/bash
#
# Service script for NaviX coordination service
# ---------------------------------------------

# Service instance name
INSTANCE="NaviX"

# Installation path
DIRPATH="$(dirname $(readlink -f $0))"

ACTION=''

DEBUG=false

while getopts 'a:d' flag; do
    case "${flag}" in
        a) ACTION="${OPTARG}" ;;
        d) DEBUG=true ;;
        *) error "Unexpected option ${flag}" ;;
    esac
done

if ${DEBUG}; then
    echo "Getting info from ${DIRPATH}"
fi

# Starting command for NaviX service
COMMAND="/usr/bin/python3.5 NaviX.py"

# Check for running service processes
if [ -f ${DIRPATH}/${INSTANCE}.pid ]; then
    PID=`/bin/cat ${DIRPATH}/${INSTANCE}.pid | /usr/bin/head -1`
fi

case "$ACTION" in

'start')
    if [ -f ${DIRPATH}/${INSTANCE}.pid ]; then
        if [ `/bin/ps -e | /bin/grep -c ${PID}` -eq 1 ]; then
            echo "Not starting ${INSTANCE} - instance already running with PID: [${PID}]"
        else
            echo "Ignoring invalid PID file"
            echo "Starting ${INSTANCE}"
            cd ${DIRPATH}/..
            if ${DEBUG}; then
                echo "Starting ${INSTANCE}-Service" > ${DIRPATH}/${INSTANCE}.log
                /usr/bin/nohup ${COMMAND} >> ${DIRPATH}/${INSTANCE}.log 2>&1 &
                echo $! > ${DIRPATH}/${INSTANCE}.pid
            else
                /usr/bin/nohup ${COMMAND} > /dev/null 2>&1 &
                echo $! > ${DIRPATH}/${INSTANCE}.pid
            fi
        fi
    else
        echo "Starting ${INSTANCE}"
        cd ${DIRPATH}/..
        if ${DEBUG}; then
            echo "Starting ${INSTANCE}-Service" > ${DIRPATH}/${INSTANCE}.log
            /usr/bin/nohup ${COMMAND} >> ${DIRPATH}/${INSTANCE}.log 2>&1 &
            echo $! > ${DIRPATH}/${INSTANCE}.pid
        else
            /usr/bin/nohup ${COMMAND} > /dev/null 2>&1 &
            echo $! > ${DIRPATH}/${INSTANCE}.pid
        fi
    fi
    ;;

'stop')
    if [ -f ${DIRPATH}/${INSTANCE}.pid ] ; then
        if [ `/bin/ps -e | /bin/grep -c ${PID}` -eq 0 ] ; then
            echo "${INSTANCE} not running. PID files invalid."
        else
            echo "Stopping ${INSTANCE}"
            # Terminating process children
            pkill -TERM -P ${PID}
            # Terminating parent process
            kill -TERM ${PID}
            /bin/rm -rf ${DIRPATH}/${INSTANCE}.pid
            if ${DEBUG}; then
                echo "Stopping ${INSTANCE}-Service" >> ${DIRPATH}/${INSTANCE}.log
            fi
        fi
    else
        echo "Cannot stop ${INSTANCE} - no PID file found!"
    fi
    ;;

'restart')
    if ${DEBUG}; then
        $0 -a stop -d
        sleep 5
        $0 -a start -d
    else
        $0 -a stop
        sleep 5
        $0 -a start
    fi
    ;;

'status')
    if [ -f ${DIRPATH}/${INSTANCE}.pid ] ; then
        if [ `/bin/ps -e | /bin/grep -c ${PID}` -eq 0 ] ; then
            echo "${INSTANCE} not running. PID files invalid."
        else
            echo "${INSTANCE} running with PID: [${PID}]"
        fi
    else
        echo "${INSTANCE} not running. No PID files found."
        exit 1
    fi
    ;;

*)
    echo "usage: $0 {-a start | stop | restart | status}"
    ;;

esac

exit 0