from unittest import TestCase
from navix.logging import *


class TestLogging(TestCase):
    maxDiff = None

    def setUp(self):
        self.thread_num = "TEST"

    def test_get_logger(self):
        # log file path
        log_file_s = 'NaviX_test.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.DEBUG)
        log = Logging()
        self.logger = log.queue_logging

        jobs = []
        how_many_tests = 10
        for i in range(how_many_tests):
            p = Process(target=self.logger.put("_INFO: TEST " + str(i)))
            jobs.append(p)
            p.start()
        p.join()
        fin = "NaviX.log"
        with open(fin, "r") as log_file:
            log_file.seek(0, 2)  # Seek @ EOF
            file_size = log_file.tell()   # Get Size
            log_file.seek(max(file_size - 1024, 0), 0)  # Set pos @ last n chars
            lines = log_file.readlines()  # Read to end
        log_file.close()
        lines = lines[-how_many_tests:]  # Get last 10 lines
        for i in range(how_many_tests):
            self.assertEqual("- INFO - TEST " + str(i) + "\n", lines[i][-len("- INFO - TEST " + str(i) + "\n"):])

    def test_configure_logger(self):
        logger_test = configure_logger()
        logger_test.error("TEST")
        x = list(logger_test.handlers)
        for i in x:
            logger_test.removeHandler(i)
            i.flush()
            i.close()
        fin = "NaviX.log"
        with open(fin, "r") as log_file:
            log_file.seek(0, 2)  # Seek @ EOF
            file_size = log_file.tell()  # Get Size
            log_file.seek(max(file_size - 1024, 0), 0)  # Set pos @ last n chars
            lines = log_file.readlines()  # Read to end
        log_file.close()
        lines = lines[-1:]  # Get last 10 lines

        logging.getLogger().removeHandler(logging.getLogger().handlers)

        self.assertEqual("- ERROR - TEST" + "\n", lines[0][-len("- ERROR - TEST" + "\n"):])
        del logger_test

    def test_configure_monitor_logger(self):
        logger_test = configure_monitor_logger()
        logger_test.info("TEST")
        x = list(logger_test.handlers)
        for i in x:
            logger_test.removeHandler(i)
            i.flush()
            i.close()
        fin = "NaviX.mon"
        with open(fin, "r") as log_file:
            log_file.seek(0, 2)  # Seek @ EOF
            file_size = log_file.tell()  # Get Size
            log_file.seek(max(file_size - 1024, 0), 0)  # Set pos @ last n chars
            lines = log_file.readlines()  # Read to end
        log_file.close()
        lines = lines[-1:]  # Get last 10 lines

        logging.getLogger().removeHandler(logging.getLogger().handlers)

        self.assertEqual("TEST" + "\n", lines[0][-len("TEST" + "\n"):])
        del logger_test

    def test_logger(self):
        mp_queue = Queue()
        log_writer = Process(target=logger, args=(mp_queue,))
        log_writer.daemon = True
        log_writer.start()
        how_many_tests = 5
        jobs=[]
        for i in range(how_many_tests):
            p = Process(target=mp_queue.put("ERROR: TEST " + str(i)))
            jobs.append(p)
            p.start()
        p.join()
        fin = "NaviX.log"
        with open(fin, "r") as log_file:
            log_file.seek(0, 2)  # Seek @ EOF
            file_size = log_file.tell()  # Get Size
            log_file.seek(max(file_size - 1024, 0), 0)  # Set pos @ last n chars
            lines = log_file.readlines()  # Read to end
        log_file.close()
        lines = lines[-how_many_tests:]  # Get last 10 lines
        for i in range(how_many_tests):
            self.assertEqual("- ERROR - TEST " + str(i) + "\n", lines[i][-len("- ERROR - TEST " + str(i) + "\n"):])

        # Clean up after test
        log_writer.terminate()

        logger_test = logging.getLogger()
        x = list(logger_test.handlers)
        for i in x:
            logger_test.removeHandler(i)
            i.flush()
            i.close()

    def test_monitor(self):
        mp_queue = Queue()
        log_writer = Process(target=monitor, args=(mp_queue,))
        log_writer.daemon = True
        log_writer.start()
        how_many_tests = 5
        jobs=[]
        for i in range(how_many_tests):
            p = Process(target=mp_queue.put("ERROR: TEST " + str(i)))
            jobs.append(p)
            p.start()
        p.join()
        fin = "NaviX.mon"
        with open(fin, "r") as log_file:
            log_file.seek(0, 2)  # Seek @ EOF
            file_size = log_file.tell()  # Get Size
            log_file.seek(max(file_size - 1024, 0), 0)  # Set pos @ last n chars
            lines = log_file.readlines()  # Read to end
        log_file.close()
        lines = lines[-how_many_tests:]  # Get last 10 lines
        for i in range(how_many_tests):
            self.assertEqual("ERROR: TEST " + str(i) + "\n", lines[i][-len("ERROR: TEST  " + str(i) + "\n"):])

        # Clean up after test
        log_writer.terminate()

        logger_test = logging.getLogger()
        x = list(logger_test.handlers)
        for i in x:
            logger_test.removeHandler(i)
            i.flush()
            i.close()





