from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings

from navix.replygenerator.check_time_stamp import remove_timestamp_from_data, \
    check_data_for_hpda_timestamp, check_timestamp


class TestCheckTimeStamp(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.INFO)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.conn = "conn"  # not good implemented
        self.logger = log.queue_logging
        return

    def test_check_timestamp(self):
        data = ("\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum."
                + "Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. "
                + "\n HPDA_TIMESTAMP = \"1524654509\"\nStet clita kasd gubergren, "
                + "no sea takimata sanctus est Lorem ipsum dolor sit amet.")
        data, timestamp = check_timestamp(self.logger, self.thread_num, data)
        self.assertEqual(data, "\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum."
                         + "Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. "
                         + "\n \nStet clita kasd gubergren, "
                         + "no sea takimata sanctus est Lorem ipsum dolor sit amet.")
        self.assertEqual(timestamp, '1524654509')

        data = ('\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum.'
                + 'Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. '
                + '\nStet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.')
        data_returned, timestamp = check_timestamp(self.logger, self.thread_num, data)
        self.assertEqual(data, data_returned)
        self.assertFalse(timestamp)
        return

    def test_remove_timestamp_from_data(self):
        data = ("\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum."
                + "Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. "
                + "\n HPDA_TIMESTAMP = \"1524654509\"\nStet clita kasd gubergren, "
                + "no sea takimata sanctus est Lorem ipsum dolor sit amet.")
        last_time = 1524654509
        data = remove_timestamp_from_data(data, last_time)
        self.assertEqual(data, "\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum."
                         + "Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. "
                         + "\n \nStet clita kasd gubergren, "
                         + "no sea takimata sanctus est Lorem ipsum dolor sit amet.")
        return

    def test_check_data_for_hpda_timestamp(self):
        data = ("\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum."
                + "Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. "
                + "\n HPDA_TIMESTAMP = \"1524654509\"\nStet clita kasd gubergren, "
                + "no sea takimata sanctus est Lorem ipsum dolor sit amet.")
        timestamp = check_data_for_hpda_timestamp(self.logger, self.thread_num, data)
        self.assertEqual(timestamp, '1524654509')

        data = ('\nerat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum.'
                + 'Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. '
                + '\nStet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.')
        timestamp = check_data_for_hpda_timestamp(self.logger, self.thread_num, data)
        self.assertFalse(timestamp)
        return
