from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings

from navix.request_pyxrootd import deeplocate_for_files_pyxrootd, stat_for_files_pyxrootd, create_pyxrootd_client


class TestRequestPyxrootd(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.INFO)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.logger = log.queue_logging
        return

    def test_deeplocate_for_files_pyxrootd(self):
        server_address = "root://eospublic.cern.ch"
        files_to_search = ["//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/"
                           "FE5A353C-9B6F-E211-8B20-00259073E398.root,root://eospublic.cern.ch//eos/"
                           "opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/"
                           "310000/FEDD2A3C-9B6F-E211-9673-485B39800BFB.root data MuRun2012B 455"]
        file_dict = deeplocate_for_files_pyxrootd(self.logger, self.thread_num,
                                                  server_address, files_to_search, 1)
        self.assertEqual({'//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/'
                          'FE5A353C-9B6F-E211-8B20-00259073E398.root,root://eospublic.cern.ch//eos/'
                          'opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/'
                          '310000/FEDD2A3C-9B6F-E211-9673-485B39800BFB.root data MuRun2012B 455':
                              ['[::188.184.50.168]:1094']}, file_dict)

        files_to_search = ["//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/"
                           "310000/FEDD2A3C-9B6F-E211-9673-485B39800BFB.root data MuRun2012B 455",
                           "//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/"
                           "E4EEC5DC-A96F-E211-9BBC-00259073E516.root"]
        file_dict = deeplocate_for_files_pyxrootd(self.logger, self.thread_num,
                                                  server_address, files_to_search, 1)
        self.assertTrue(file_dict == ({'//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/'
                                      'E4EEC5DC-A96F-E211-9BBC-00259073E516.root': ['[::188.184.50.168]:1094'],
                                       '//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/'
                                       'FEDD2A3C-9B6F-E211-9673-485B39800BFB.root data MuRun2012B 455':
                                           ['[::188.184.50.168]:1094']}
                                      or {'//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/'
                                          'FEDD2A3C-9B6F-E211-9673-485B39800BFB.root data MuRun2012B 455':
                                              ['[::188.184.50.168]:1094'],
                                          '//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/310000/'
                                          'E4EEC5DC-A96F-E211-9BBC-00259073E516.root': ['[::188.184.50.168]:1094']}))
        server_address = "root://cms-xrd-global.cern.ch"
        files_to_search = ["/store/test/xrootd/T1_DE_KIT/store/mc/SAM/GenericTTbar/AODSIM/"
                           + "CMSSW_9_0_0_90X_mcRun1_realistic_v4-v1/10000/28B9D1FB-8B31-E711-AA4E-0025905B85B2.root21"]
        file_dict = deeplocate_for_files_pyxrootd(self.logger, self.thread_num,
                                                  server_address, files_to_search, 1)
        self.assertEqual({'/store/test/xrootd/T1_DE_KIT/store/mc/SAM/GenericTTbar/'
                          + 'AODSIM/CMSSW_9_0_0_90X_mcRun1_realistic_v4-v1/10000/'
                          + '28B9D1FB-8B31-E711-AA4E-0025905B85B2.root21': ['No server']}, file_dict)

    def test_stat_for_files_pyxrootd(self):
        server_address = "root://cms-xrd-global.cern.ch"
        file = ("/store/test/xrootd/T1_DE_KIT/store/mc/SAM/GenericTTbar/AODSIM/"
                + "CMSSW_9_0_0_90X_mcRun1_realistic_v4-v1/10000/28B9D1FB-8B31-E711-AA4E-0025905B85B2.root")
        client = create_pyxrootd_client(self.logger, self.thread_num, server_address)
        status, stat = stat_for_files_pyxrootd(client, file)
        del client
        self.assertEqual("[SUCCESS] ", str(status))
        self.assertEqual(stat.size, 1185995054)
        file = ("/store/test/xrootd/T1_DE_KIT/store/mc/SAM/GenericTTbar/AODSIM/"
                + "CMSSW_9_0_0_90X_mcRun1_realistic_v4-v1/10000/28B9D1FB-8B31-E711-AA4E-0025905B85B2.root22")
        client = create_pyxrootd_client(self.logger, self.thread_num, server_address)
        status, stat = stat_for_files_pyxrootd(client, file)
        del client
        self.assertEqual("[ERROR] Server responded with an error: [3011] No servers are available to read the file.\n",
                         str(status))
        with self.assertRaises(Exception) as err:
            self.assertEqual(stat.size, None)
